#ifndef NODEPP_TIMER
#define NODEPP_TIMER

/*────────────────────────────────────────────────────────────────────────────*/

#include "generator.h"

/*────────────────────────────────────────────────────────────────────────────*/

namespace nodepp { namespace timer {
    
    template< class V, class... T >
    ptr_t<int> add ( V func, ulong* time, const T&... args ){
        ptr_t<ulong> stm = new ulong( process::millis() + *time ); 
        auto         prs = _timer_::timer();
        ptr_t<int>   out = new int( 1 ); 
        process::task::add( prs, func, out, stm, time, args... ); 
        return out;
    };
    
    template< class V, class... T >
    ptr_t<int> add ( V func, ulong time, const T&... args ){
        ptr_t<ulong> stm = new ulong( process::millis() + time ); 
        auto         prs = _timer_::timer();
        ptr_t<int>   out = new int( 1 ); 
        process::task::add( prs, func, out, time, stm, args... ); 
        return out;
    };
    
    /*─······································································─*/

    template< class V, class... T >
    ptr_t<int> timeout ( V func, ulong* time, const T&... args ){
        return timer::add([=]( T... args ){ func(args...); return -1; }, time, args... );
    };

    template< class V, class... T >
    ptr_t<int> timeout ( V func, ulong time, const T&... args ){
        return timer::add([=]( T... args ){ func(args...); return -1; }, time, args... );
    };
    
    /*─······································································─*/

    template< class V, class... T >
    ptr_t<int> interval ( V func, ulong* time, const T&... args ){
        return timer::add([=]( T... args ){ func(args...); return 1; }, time, args... );
    };

    template< class V, class... T >
    ptr_t<int> interval( V func, ulong time, const T&... args ){
        return timer::add([=]( T... args ){ func(args...); return 1; }, time, args... );
    };
    
    /*─······································································─*/
    
    void await( ulong* time ){
        ulong stamp = process::millis() + *time;
        while( process::millis() < stamp )
             { process::next(); }
    };

    void await( ulong time ){ await( (ulong*) &time ); }
    
    /*─······································································─*/

    void clear( const ptr_t<int>& address ){ if( !address ) *address = 0; }

}}

/*────────────────────────────────────────────────────────────────────────────*/

namespace nodepp { namespace utimer {
    
    template< class V, class... T >
    ptr_t<int> add ( V func, ulong* time, const T&... args ){
        ptr_t<ulong> stm = new ulong( process::micros() + time );  
        auto         prs = _timer_::utimer();
        ptr_t<int>   out = new int( 1 );  
        process::task::add( prs, func, out, stm, args... ); 
        return out;
    };
    
    template< class V, class... T >
    ptr_t<int> add ( V func, ulong time, const T&... args ){
        ptr_t<ulong> stm = new ulong( process::micros() + time );  
        auto         prs = _timer_::utimer();
        ptr_t<int>   out = new int( 1 );  
        process::task::add( prs, func, out, stm, args... ); 
        return out;
    };
    
    /*─······································································─*/

    template< class V, class... T >
    ptr_t<int> timeout ( V func, ulong* time, const T&... args ){
        return utimer::add([=]( T... args ){ func(args...); return -1; }, time, args... );
    };

    template< class V, class... T >
    ptr_t<int> timeout ( V func, ulong time, const T&... args ){
        return utimer::add([=]( T... args ){ func(args...); return -1; }, time, args... );
    };
    
    /*─······································································─*/

    template< class V, class... T >
    ptr_t<int> interval ( V func, ulong* time, const T&... args ){
        return utimer::add([=]( T... args ){ func(args...); return 1; }, time, args... );
    };

    template< class V, class... T >
    ptr_t<int> interval( V func, ulong time, const T&... args ){
        return utimer::add([=]( T... args ){ func(args...); return 1; }, time, args... );
    };
    
    /*─······································································─*/
    
    void await( ulong* time ){
        ulong stamp = process::micros() + *time;
        while( process::micros() < stamp )
             { process::next(); }
    };

    void await( ulong time ){ await( (ulong*) &time ); }
    
    /*─······································································─*/

    void clear( const ptr_t<int>& address ){ if( !address ) *address = 0; }

}}

/*────────────────────────────────────────────────────────────────────────────*/

#endif
